﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Globalization;
using System.Windows.Forms;

namespace RSTextMonitor
{
    class ScreenCapture
    {
        public int ScreenWidth = 800;
        public int ScreenHeight = 600;
        public int OffsetX = 0;
        public int OffsetY = 0;

        public ScreenCapture()
        {
            Screen[] Screens = Screen.AllScreens;
            int ScreenCount = Screens.Length;
            int LowestX = 0, HighestX = 0, LowestY = 0, HighestY = 0;

            for (int i = 0; i < ScreenCount; i++)
            {
                LowestX = Math.Min(LowestX, Screens[i].Bounds.X);
                HighestX = Math.Max(HighestX, Screens[i].Bounds.X + Screens[i].Bounds.Width);

                LowestY = Math.Min(LowestY, Screens[i].Bounds.Y);
                HighestY = Math.Max(HighestY, Screens[i].Bounds.Y + Screens[i].Bounds.Height);
            }

            this.ScreenWidth = HighestX - LowestX;
            this.ScreenHeight = HighestY - LowestY;
            this.OffsetX = LowestX;
            this.OffsetY = LowestY;
        }

        public Color GetColorOfPx(int x, int y)
        {
            Bitmap bmp = CaptureScreenArea(x, y, 1, 1);
            Color answer = bmp.GetPixel(0, 0);
            bmp.Dispose();
            return answer;
        }

        public static Bitmap CapturePrimaryScreen()
        {
            return CaptureWindow(User32.GetDesktopWindow());
        }

        public static Bitmap CaptureWindow(IntPtr handle)
        {
            IntPtr hdcSrc = User32.GetWindowDC(handle);
            User32.RECT windowRect = new User32.RECT();
            User32.GetWindowRect(handle, ref windowRect);
            int width = windowRect.right - windowRect.left;
            int height = windowRect.bottom - windowRect.top;
            IntPtr hdcDest = GDI32.CreateCompatibleDC(hdcSrc);
            IntPtr hBitmap = GDI32.CreateCompatibleBitmap(hdcSrc, width, height);
            IntPtr hOld = GDI32.SelectObject(hdcDest, hBitmap);
            GDI32.BitBlt(hdcDest, 0, 0, width, height, hdcSrc, 0, 0, GDI32.SRCCOPY);
            GDI32.SelectObject(hdcDest, hOld);
            GDI32.DeleteDC(hdcDest);
            User32.ReleaseDC(handle, hdcSrc);
            Bitmap bmp = Bitmap.FromHbitmap(hBitmap);
            GDI32.DeleteObject(hBitmap);
            return bmp;
        }

        public Bitmap CaptureAllScreens()
        {
            return CaptureScreenArea(0, 0, ScreenWidth, ScreenHeight);
        }

        public Bitmap CaptureScreenArea(int X, int Y, int Width, int Height)
        {
            IntPtr hdcSrc = GDI32.CreateDC("DISPLAY", null, null, IntPtr.Zero);
            IntPtr hdcDest = GDI32.CreateCompatibleDC(hdcSrc);
            IntPtr hBitmap = GDI32.CreateCompatibleBitmap(hdcSrc, Width, Height);
            GDI32.SelectObject(hdcDest, hBitmap);
            GDI32.BitBlt(hdcDest, 0, 0, Width, Height, hdcSrc, OffsetX + X, OffsetY + Y,
                        0x40000000 | 0x00CC0020); //SRCCOPY AND CAPTUREBLT
            Bitmap bmpret = Bitmap.FromHbitmap(hBitmap);
            GDI32.DeleteDC(hdcSrc);
            GDI32.DeleteDC(hdcDest);
            GDI32.DeleteObject(hBitmap);
            return bmpret;
        }

        public static void CaptureWindowToFile(string filename)
        {
            Bitmap img = CaptureWindow(User32.GetForegroundWindow());
            img.Save(filename);
        }

        public static void CaptureScreenToFile(string filename)
        {
            Bitmap img = CapturePrimaryScreen();
            img.Save(filename);
        }

        public void CaptureAllScreensToFile(string filename)
        {
            Bitmap img = CaptureAllScreens();
            img.Save(filename);
        }

        private class GDI32
        {
            public const int SRCCOPY = 0x00CC0020; // BitBlt dwRop parameter
            [DllImport("gdi32.dll")]
            public static extern bool BitBlt(IntPtr hObject, int nXDest, int nYDest,
                int nWidth, int nHeight, IntPtr hObjectSource,
                int nXSrc, int nYSrc, int dwRop);
            [DllImport("gdi32.dll")]
            public static extern IntPtr CreateCompatibleBitmap(IntPtr hDC, int nWidth, int nHeight);
            [DllImport("gdi32.dll")]
            public static extern IntPtr CreateCompatibleDC(IntPtr hDC);
            [DllImport("gdi32.dll")]
            public static extern bool DeleteDC(IntPtr hDC);
            [DllImport("gdi32.dll")]
            public static extern bool DeleteObject(IntPtr hObject);
            [DllImport("gdi32.dll")]
            public static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);
            [DllImport("gdi32.dll")]
            public static extern IntPtr CreateDC(string lpszDriver, string lpszDevice, string lpszOutput, IntPtr lpInitData);
            [DllImport("GDI32.dll")]
            public static extern int GetDeviceCaps(int hdc, int nIndex);
        }

        private class User32
        {
            [StructLayout(LayoutKind.Sequential)]
            public struct RECT
            {
                public int left;
                public int top;
                public int right;
                public int bottom;
            }
            [DllImport("user32.dll")]
            public static extern IntPtr GetDesktopWindow();
            [DllImport("user32.dll")]
            public static extern IntPtr GetWindowDC(IntPtr hWnd);
            [DllImport("user32.dll")]
            public static extern IntPtr ReleaseDC(IntPtr hWnd, IntPtr hDC);
            [DllImport("user32.dll")]
            public static extern IntPtr GetWindowRect(IntPtr hWnd, ref RECT rect);
            [DllImport("User32.dll")]
            public static extern IntPtr GetForegroundWindow();
        }
    }
}
